/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2007        */
/*-----------------------------------------------------------------------*/
/* This is a stub disk I/O module that acts as front end of the existing */
/* disk I/O modules and attach it to FatFs module with common interface. */
/*-----------------------------------------------------------------------*/
#include <string.h>
#include "diskio.h"
#include "ch32v30x.h"
#include "w25q.h"

/* Ϊÿ豸һ */
#define ATA			           1     // SD
#define SPI_FLASH		       0     // ԤⲿSPI Flashʹ

/*-----------------------------------------------------------------------*/
/* ȡ豸״̬                                                          */
/*-----------------------------------------------------------------------*/
DSTATUS disk_status (
	BYTE pdrv		/*  */
)
{
	DSTATUS status = STA_NOINIT;
	
	switch (pdrv) {
		case ATA:	/* SD CARD */
			status &= ~STA_NOINIT;
			break;
    
		case SPI_FLASH:        /* SPI Flash */
		    status &= ~STA_NOINIT;
			break;

		default:
			status = STA_NOINIT;
	}
	return status;
}

/*-----------------------------------------------------------------------*/
/* 豸ʼ                                                            */
/*-----------------------------------------------------------------------*/
DSTATUS disk_initialize (
	BYTE pdrv				/*  */
)
{
	DSTATUS status = STA_NOINIT;	
	switch (pdrv) {
		case ATA:	         /* SD CARD */
		
			break;
    
		case SPI_FLASH:    /* SPI Flash */ 
		    SPI_Flash_Init();
		    if(SPI_Flash_ReadID() == W25Q128) status &= ~STA_NOINIT;
		    else status = STA_NOINIT;

			break;
      
		default:
			status = STA_NOINIT;
	}
	return status;
}


/*-----------------------------------------------------------------------*/
/* ȡݵָ洢                                              */
/*-----------------------------------------------------------------------*/
DRESULT disk_read (
	BYTE pdrv,		/* 豸(0..) */
	BYTE *buff,		/* ݻ */
	DWORD sector,	/* ׵ַ */
	UINT count		/* (1..128) */
)
{
	DRESULT status = RES_PARERR;
//	SD_Error SD_state = SD_RESPONSE_NO_ERROR;
	
	switch (pdrv) {
		case ATA:	/* SD CARD */						
			
//			SD_state=SD_ReadMultiBlocks(buff,(uint64_t)sector*SD_BLOCKSIZE,SD_BLOCKSIZE,count);
//
//			if(SD_state!=SD_RESPONSE_NO_ERROR)
//				status = RES_PARERR;
//		  else
//			  status = RES_OK;
			break;   
			
		case SPI_FLASH:
		    SPI_Flash_Read(buff, sector*4096, count*4096);
		    status = RES_OK;
		break;
    
		default:
			status = RES_PARERR;
	}
	return status;
}



/*-----------------------------------------------------------------------*/
/* ддָռ                                      */
/*-----------------------------------------------------------------------*/
#if _USE_WRITE
DRESULT disk_write (
	BYTE pdrv,			  /* 豸(0..) */
	const BYTE *buff,	/* дݵĻ */
	DWORD sector,		  /* ׵ַ */
	UINT count			  /* (1..128) */
)
{
	DRESULT status = RES_PARERR;
//	SD_Error SD_state = SD_RESPONSE_NO_ERROR;
	
	if (!count) {
		return RES_PARERR;		/* Check parameter */
	}

	switch (pdrv) {
		case ATA:	/* SD CARD */  
		
//			SD_state=SD_WriteMultiBlocks((uint8_t *)buff,(uint64_t)sector*SD_BLOCKSIZE,SD_BLOCKSIZE,count);
//
//			if(SD_state!=SD_RESPONSE_NO_ERROR)
//				status = RES_PARERR;
//		  else
//			  status = RES_OK;
		break;

		case SPI_FLASH:
		    SPI_Flash_Write((unsigned char *)buff, sector*4096, count*4096);
		    status = RES_OK;
		break;
    
		default:
			status = RES_PARERR;
	}
	return status;
}
#endif


/*-----------------------------------------------------------------------*/
/*                                                               */
/*-----------------------------------------------------------------------*/

#if _USE_IOCTL
DRESULT disk_ioctl (
	BYTE pdrv,		/*  */
	BYTE cmd,		  /* ָ */
	void *buff		/* д߶ȡݵַָ */
)
{
	DRESULT status = RES_PARERR;
	switch (pdrv) {
		case ATA:	/* SD CARD */
			break;
    
		case SPI_FLASH:
            switch (cmd)
            {
                // Get R/W sector size (WORD)
                case GET_SECTOR_SIZE :
                    *(WORD * )buff = 4096;
                break;
                // Get erase block size in unit of sector (DWORD)
                case GET_BLOCK_SIZE :
                    *(DWORD * )buff = 1;
                break;

                case GET_SECTOR_COUNT:
                    *(DWORD * )buff = 4096;
                    break;
                case CTRL_SYNC :
                break;
            }
		break;
    
		default:
			status = RES_PARERR;
	}
	status = RES_OK;
	return status;
}
#endif

							 
DWORD get_fattime(void) {
	/* صǰʱ */
	return	  ((DWORD)(2015 - 1980) << 25)	/* Year 2015 */
			| ((DWORD)1 << 21)				/* Month 1 */
			| ((DWORD)1 << 16)				/* Mday 1 */
			| ((DWORD)0 << 11)				/* Hour 0 */
			| ((DWORD)0 << 5)				  /* Min 0 */
			| ((DWORD)0 >> 1);				/* Sec 0 */
}

