#include "eth.h"
#include "string.h"
#include "stdio.h"
#include "uart_debug.h"

#define DESADDR "www.wch.cn"
#define DESPORT 8999

u8 MACAddr[6];                                    //MAC address
u8 IPAddr[4]   = {0, 0, 0, 0};                    //IP address
u8 GWIPAddr[4] = {0, 0, 0, 0};                    //Gateway IP address
u8 IPMask[4]   = {0, 0, 0, 0};                    //subnet mask
//u8 DESIP[4]    = {192, 168, 1, 9};              //destination IP address
u8 DESIP[4]    = {47, 240, 94, 152};
u16 desport = 80;                               //destination port
u16 srcport = 1000;                               //source port

unsigned char router_con_status = 0;
unsigned char tcp_con_status = 0;

u8 SocketId;
u8 SocketRecvBuf[WCHNET_MAX_SOCKET_NUM][RECE_BUF_LEN];  //socket receive buffer

unsigned char tcp_is_recv = 0;
u8 tcp_recv_buff[WCHNET_TCP_MSS*4];
uint32_t tcp_recv_len;

/*********************************************************************
 * @fn      mStopIfError
 *
 * @brief   check if error.
 *
 * @param   iError - error constants.
 *
 * @return  none
 */
void mStopIfError(u8 iError)
{
    if (iError == WCHNET_ERR_SUCCESS) return;
    printf("Error: %02X\r\n", (u16) iError);
}

/*********************************************************************
 * @fn      TIM2_Init
 *
 * @brief   Initializes TIM2.
 *
 * @return  none
 */
void TIM2_Init(void)
{
    TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure = {0};

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

    TIM_TimeBaseStructure.TIM_Period = SystemCoreClock / 1000000;
    TIM_TimeBaseStructure.TIM_Prescaler = WCHNETTIMERPERIOD * 1000 - 1;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

    TIM_Cmd(TIM2, ENABLE);
    TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
    NVIC_SetPriority(TIM2_IRQn, 0x80);
    NVIC_EnableIRQ(TIM2_IRQn);
}

/*********************************************************************
 * @fn      WCHNET_CreateTcpSocket
 *
 * @brief   Create TCP Socket
 *
 * @return  none
 */
void WCHNET_CreateTcpSocket(void)
{
    u8 i;
    SOCK_INF TmpSocketInf;

    memset((void *) &TmpSocketInf, 0, sizeof(SOCK_INF));
    memcpy((void *) TmpSocketInf.IPAddr, DESIP, 4);
    TmpSocketInf.DesPort = desport;
    TmpSocketInf.SourPort = srcport++;
    TmpSocketInf.ProtoType = PROTO_TYPE_TCP;
    TmpSocketInf.RecvBufLen = RECE_BUF_LEN;
    i = WCHNET_SocketCreat(&SocketId, &TmpSocketInf);
    printf("SocketId %d\r\n", SocketId);

    struct _KEEP_CFG cfg;
    cfg.KLIdle = 20000;
    cfg.KLIntvl = 5000;
    cfg.KLCount = 3;
    WCHNET_ConfigKeepLive(&cfg);
    WCHNET_SocketSetKeepLive(SocketId, ENABLE);

    mStopIfError(i);
    i = WCHNET_SocketConnect(SocketId);                        //make a TCP connection
    mStopIfError(i);
}

void WCHNET_Receive()
{
    tcp_is_recv = 1;
    tcp_recv_len = WCHNET_SocketRecvLen(SocketId, NULL);
    WCHNET_SocketRecv(SocketId, tcp_recv_buff, &tcp_recv_len);
    tcp_recv_buff[tcp_recv_len] = '\0';
}

void Eth_Send(unsigned char *data, uint32_t len)
{
    WCHNET_SocketSend(SocketId, data, &len);
}

unsigned char Eth_Receive()
{
    return tcp_is_recv;
}

unsigned int Eth_Receive_Len()
{
    return tcp_recv_len;
}

unsigned char *Eth_Receive_Data()
{
    return tcp_recv_buff;
}

void Eth_Receive_Clear()
{
    tcp_is_recv = 0;
}


/*********************************************************************
 * @fn      WCHNET_HandleSockInt
 *
 * @brief   Socket Interrupt Handle
 *
 * @param   socketid - socket id.
 *          intstat - interrupt status
 *
 * @return  none
 */
void WCHNET_HandleSockInt(u8 socketid, u8 intstat)
{
    if (intstat & SINT_STAT_RECV)                               //receive data
    {
        WCHNET_Receive();                          //Data loopback
    }
    if (intstat & SINT_STAT_CONNECT)                            //connect successfully
    {
        WCHNET_ModifyRecvBuf(socketid, (u32) SocketRecvBuf[socketid], RECE_BUF_LEN);
        tcp_con_status = 1;
        printf("TCP Connect Success\r\n");
        unsigned char temp_str[] = "GET /cyc1/cyc.tft HTTP/1.1\r\nHOST: 47.240.94.152\r\n\r\n";
        Eth_Send(temp_str, strlen(temp_str));
    }
    if (intstat & SINT_STAT_DISCONNECT)                         //disconnect
    {
        printf("TCP Disconnect\r\n");
    }
    if (intstat & SINT_STAT_TIM_OUT)                            //timeout disconnect
    {
        printf("TCP Timeout\r\n");
        WCHNET_CreateTcpSocket();
    }
}

/*********************************************************************
 * @fn      WCHNET_HandleGlobalInt
 *
 * @brief   Global Interrupt Handle
 *
 * @return  none
 */
void WCHNET_HandleGlobalInt(void)
{
    u8 intstat;
    u16 i;
    u8 socketint;

    intstat = WCHNET_GetGlobalInt();                              //get global interrupt flag
    if (intstat & GINT_STAT_UNREACH)                              //Unreachable interrupt
    {
        printf("GINT_STAT_UNREACH\r\n");
    }
    if (intstat & GINT_STAT_IP_CONFLI)                            //IP conflict
    {
        printf("GINT_STAT_IP_CONFLI\r\n");
    }
    if (intstat & GINT_STAT_PHY_CHANGE)                           //PHY status change
    {
        i = WCHNET_GetPHYStatus();
        if (i & PHY_Linked_Status)
        {
            printf("PHY Link Success\r\n");
//            WCHNET_DHCPStop();
//            WCHNET_DHCPStart(WCHNET_DHCPCallBack);
        }
        else
        {
            router_con_status = 0;
        }
    }
    if (intstat & GINT_STAT_SOCKET) {                             //socket related interrupt
        for (i = 0; i < WCHNET_MAX_SOCKET_NUM; i++) {
            socketint = WCHNET_GetSocketInt(i);
            if (socketint)
                WCHNET_HandleSockInt(i, socketint);
        }
    }
}


void WCHNET_DNSCallBack(const char *name, u8 *ipaddr, void *callback_arg)
{
    if(ipaddr == NULL)
    {
        printf("DNS Fail\r\n");
        return;
    }
    printf("Host Name: %s\r\n", name);
    printf("IP: %d.%d.%d.%d\r\n", ipaddr[0], ipaddr[1], ipaddr[2], ipaddr[3]);
    if(callback_arg != NULL)
    {
        printf("callback_arg: %02x\r\n", (*(u8 *)callback_arg));
    }
    WCHNET_DNSStop();                                                          //stop DNS,and release socket
}


/*********************************************************************
 * @fn      WCHNET_DHCPCallBack
 *
 * @brief   DHCPCallBack
 *
 * @param   status - status returned by DHCP
 *                   0x00 - Success
 *                   0x01 - Failure
 *          arg - Data returned by DHCP
 *
 * @return  DHCP status
 */
u8 WCHNET_DHCPCallBack(u8 status, void *arg)
{
    u8 *p;

    if(!status)
    {
        p = arg;
        printf("DHCP Success\r\n");
        memcpy(IPAddr, p, 4);
        memcpy(GWIPAddr, &p[4], 4);
        memcpy(IPMask, &p[8], 4);
        printf("IPAddr: %d.%d.%d.%d \r\n", (u16)IPAddr[0], (u16)IPAddr[1], (u16)IPAddr[2], (u16)IPAddr[3]);
        printf("GWIPAddr: %d.%d.%d.%d \r\n", (u16)GWIPAddr[0], (u16)GWIPAddr[1], (u16)GWIPAddr[2], (u16)GWIPAddr[3]);
        printf("IPMask: %d.%d.%d.%d \r\n", (u16)IPMask[0], (u16)IPMask[1], (u16)IPMask[2], (u16)IPMask[3]);
        printf("DNS1: %d.%d.%d.%d \r\n", p[12], p[13], p[14], p[15]);            //DNS server address provided by the router
        WCHNET_CreateTcpSocket();
//        WCHNET_CreateTcpSocket();
//        WCHNET_InitDNS(&p[12], 53);                                         //Set DNS server IP address, and DNS server port is 53
//        WCHNET_HostNameGetIp("www.wch.cn", DESIP, WCHNET_DNSCallBack, NULL);  //Start DNS
        router_con_status = 1;
        return READY;
    }
    else
    {
        printf("DHCP Fail %02x \r\n", status);
        return NoREADY;
    }
}

unsigned char Router_Con_Status()
{
    return router_con_status;
}

unsigned char TCP_Con_Status()
{
    return tcp_con_status;
}

